#pragma once
#include <initializer_list>
#include <iostream>

namespace linalg {
    class Matrix {
    private:
        double* m_ptr;
        int m_rows;
        int m_columns;

    public:
        Matrix();
        Matrix(int rows);
        Matrix(int rows, int columns);
        Matrix(const Matrix& other);
        Matrix(Matrix&& other) noexcept;
        Matrix(std::initializer_list<std::initializer_list<double>> init_list);
        template <typename T>
        Matrix(std::initializer_list<T> init_list) {
            m_rows = 1;
            m_columns = init_list.size();
            if (m_columns == 0) {
                throw std::runtime_error("init_list не может быть пустым.");
            }
            m_ptr = new double[m_columns];
            int i = 0;
            for (const auto& val : init_list) {
                m_ptr[i] = static_cast<double>(val);
                i++;
            }
        }
        ~Matrix();

        int rows() const;
        int columns() const;
        bool empty() const;
        void reshape(int rows, int columns);
        friend std::ostream& operator<<(std::ostream& os, const Matrix& matrix);
        Matrix& operator=(const Matrix& other);
        Matrix& operator=(Matrix&& other) noexcept;
        double& operator()(int row, int col);
        const double& operator()(int row, int col) const;

        Matrix operator+(const Matrix &other) const;
        Matrix &operator+=(const Matrix &other);

        Matrix operator-(const Matrix &other) const;
        Matrix &operator-=(const Matrix &other);

        Matrix operator*(const Matrix &other) const;
        Matrix operator*(double scalar) const;
        friend Matrix operator*(double scalar, const Matrix &matrix);
        Matrix &operator*=(const Matrix &other);
        Matrix &operator*=(double scalar);

        bool operator==(const Matrix &other) const;
        bool operator!=(const Matrix &other) const;

        double norm() const;
        double trace() const;
    };
}